#!/usr/bin/env node
import { promisify } from "util";
import cp from "child_process";
import path from "path";
import fs from "fs";
import chalk from "chalk";
import inquirer from "inquirer";

// convert libs to promises
const exec = promisify(cp.exec);
const rm = promisify(fs.rm);

const art = `
              ^^^^^^^^^^^:     ~5B&@@@#P7                              JGGJ      :???????????: ^????
           7GY!?5555555555?:  J@@@#PPB@@@P. ..  .:.   .:.      .::.    P@@5 .:.   7YYY555YYY7   ?555
         7B@@@&Y!?Y555555555?^G@@G.   Y@@@!7&#BG&&&G?P&&&BY. J#&&&&BY: 5@@#B&&&B7  . ~555^ .    7555
      .7B@@@@@@@&5!?5555555555JJ#Y    7@@@7?@@@575@@@B?J&@@Y ^Y?7?B@@P 5@@&Y7Y@@@~   ^555^      7555
    .7B@@@@@@@@@&5: ^?55555555JJ#G.   Y@@@!?@@B  .&@@!  5@@P.YBBGPB@@#.5@@P   #@@7   ^555^      7555
  .?#@@@@@@@@@&Y:     ^?555Y?:7@@@#GG#@@@5 ?@@G  :&@@~  5@@57@@&!~P@@#.5@@5  .#@@7   ^555^      7555
 !B@@@@@@@@@&5^         ^J?:   ^YB&@@&B5!  7B5^  .B##~  J##Y JB##BYB#G.Y##Y   G##!   ^YY?.      !5YY
`;

// Split the art into lines
const lines = art.split("\n");

// Print each line using chalk
lines.forEach((line) => {
  console.log(chalk.hex("#F08619")(line));
});

//? Start installer
console.log(chalk.yellow("Omahti CMS Web Starter initiating... \n"));

//? Check if the project name is provided
if (process.argv.length < 3) {
  console.log(chalk.red("ERROR: Missing project name"));
  console.log("You have to provide a name to this new project.");
  console.log("For example:");
  console.log(chalk.underline("npx omahti-cms-web-starter new-project"));
  console.log(chalk.yellow("Exiting... \n"));

  process.exit(1);
}

const projectName = process.argv[2];
const currentPath = process.cwd();
const projectPath = path.join(currentPath, projectName);

const git_repo = "https://github.com/OmahTI-UGM/omahti-cms-web-starter.git";

//? Check if the project already exists
if (fs.existsSync(projectPath)) {
  console.log(chalk.red("ERROR: Project already exists"));
  console.log(
    `The file ${projectName} already exist in the current directory, please give it another name.`,
  );
  console.log(chalk.yellow("Exiting... \n"));

  process.exit(1);
} else {
  fs.mkdirSync(projectPath);
}

//? Main Setup
try {
  //? Ask for additional configurations
  inquirer
    .prompt([
      {
        name: "useSmoothScroll",
        message: "Do you want to use smooth scrolling feature (Lenis) (y/n)?",
      },
      {
        name: "cmaToken",
        message: "Enter your Contentful Management API token (optional):",
      },
      {
        name: "spaceID",
        message: "Enter your Contentful space ID (optional):",
      },
    ])
    .then(async (answers) => {
      //? Clone the boilerplate
      console.log(chalk.whiteBright("Cloning the boilerplate..."));
      await exec(`git clone ${git_repo} ${projectName} --quiet`);

      //? Remove unnecessary files
      console.log(chalk.whiteBright("Cleaning up..."));
      const rmGit = rm(path.join(projectPath, ".git"), {
        recursive: true,
        force: true,
      });
      const rmBin = rm(path.join(projectPath, "bin"), {
        recursive: true,
        force: true,
      });

      //? Remove smooth-scroll-wrapper.tsx if not needed
      if (!answers.useSmoothScroll) {
        //? Remove smooth-scroll-wrapper.tsx
        const rmSmoothScroll = rm(
          path.join(projectPath, "components", "smooth-scroll-wrapper.tsx"),
          {
            recursive: true,
            force: true,
          },
        );

        //? Modify app/layout.tsx
        const layoutPath = path.join(projectPath, "pages", "_app.tsx");
        const layoutFile = fs.readFileSync(layoutPath, "utf8");
        const updatedLayoutFile = layoutFile
          .replace(
            /import SmoothScrollWrapper from "@\/components\/smooth-scroll-wrapper";/g,
            "",
          )
          .replace(
            /<SmoothScrollWrapper>{children}<\/SmoothScrollWrapper>/g,
            "{children}",
          );
        fs.writeFileSync(layoutPath, updatedLayoutFile);

        //? Modify package.json
        const packagePath = path.join(projectPath, "package.json");
        const packageFile = fs.readFileSync(packagePath, "utf8");
        const packageFileUpdated = packageFile
          .replace(
            /"name": "omahti-cms-web-starter"/g,
            `"name": "${projectName}"`,
          )
          .replace(/"@studio-freight\/lenis": "\^1\.0\.37",\n/g, "")
          .replace(/"chalk": "\^5\.3\.0",\n/g, "");

        fs.writeFileSync(packagePath, packageFileUpdated);

        await Promise.all([rmGit, rmBin, rmSmoothScroll]);
      } else {
        //? Modify package.json
        const packagePath = path.join(projectPath, "package.json");
        const packageFile = fs.readFileSync(packagePath, "utf8");
        const packageFileUpdatedName = packageFile.replace(
          /"name": "omahti-cms-web-starter"/g,
          `"name": "${projectName}"`,
        );
        const packageFileUpdatedDeps = packageFileUpdatedName.replace(
          /"chalk": "\^5\.3\.0",\n/g,
          "",
        );
        fs.writeFileSync(packagePath, packageFileUpdatedDeps);

        await Promise.all([rmGit, rmBin]);
      }

      //? Install dependencies
      process.chdir(projectPath);
      console.log(chalk.whiteBright("Installing dependencies..."));
      await exec("npm install");

      //? Configure Contentful
      console.log(chalk.whiteBright("Configuring Contentful..."));
      await exec(
        "npx contentful space export --config contentful/export-config.json --management-token " +
          answers.cmaToken +
          " --space-id " +
          answers.spaceID,
      );
      await exec(
        "npx cf-content-types-generator contentful/export.json -X --out contentful/types",
      );

      //? Finishing up
      console.log(chalk.green("Installation completed!"));
      console.log("You can now run your app with:");
      console.log(`    cd ${projectName}`);
      console.log(`    npm run dev`);
      console.log(chalk.yellow("OmahTI - we made IT for everyone!"));
    });
} catch (error) {
  fs.rmSync(projectPath, { recursive: true, force: true });
  console.log(error);
}
